
.equ	color,0
.equ	FRAME,#0b00010000		; boja za auto adjustment
.equ	specsprites,#17			; specijalni sprajtovi koji ne zaklanjaju ostale (17,18,19,20)

.macro	horsync
1:
	btss	IFS0,#T1IF	; test horizontal sync interrupt flag
	bra	1b				; wait for horizontal sync
	mov	#14,w0			; minuend
	subr	TMR1,WREG	; 14-TMR1
	repeat	w0
	nop					; repeat 14-TMR1 times until TMR1 is exactly 14
	bclr	IFS0,#T1IF	; reset horizontal sync interrupt flag
.endm

.macro	defsound
	mov	#0b10101010,w1
	cp0.b	jackport
	ifnz
	mov	w1,U1TXREG	; *** put default sound data byte in UART TX
	cp0.b	effport
	ifnz
	mov	w1,U2TXREG	; *** put default sound data byte in UART TX
.endm

;***********************************************************************************
;****************************                          *****************************
;****************************    TIMER 2 INTERRUPT     *****************************
;****************************    0: VERT SYNC          *****************************
;****************************    1: MARKERS            *****************************
;****************************    2: VGA FRAME          *****************************
;****************************    3: BOTTOM TEXT        *****************************
;****************************                          *****************************
;***********************************************************************************

.global __T2Interrupt
__T2Interrupt:			; TIMER2 interrupt, vert sync
	push.s		; \

	cp0	VGATASK		; 0=vert sync, 1=markers, 2=main frame, 3=bottom text
	bra	z,vertsync
	dec	VGATASK,WREG
	bra	z,taskmarkers
	dec	w0,w0
	bra	z,taskvga
	bra	tasktext

vertsync:
	setvsync		; __/^^

	mov	#32*MARKERPOS-1,w0	; VREME OD <VERTSYNC> DO <MARKERS>
	mov	w0,PR2
	inc	VGATASK

	bclr	IFS0,#T1IF

	defsound

	horsync			; vert sync iznosi 2 linije
	horsync			; vert sync iznosi 2 linije
	clrvsync		; ^^\__

	defsound
t2exit:
	pop.s		; /
	bclr	IFS0,#T2IF	; clr int flag
	retfie

;***********************************************************************************
;****************************                          *****************************
;****************************          MARKERS         *****************************
;****************************                          *****************************
;***********************************************************************************

taskmarkers:			; gornji okvir za auto adjustment
;				  ovde su markeri (task 0)
	mov	#32*(FRAMEPOS-MARKERPOS)-1,w0	; VREME OD <MARKER> do <FRAME>
	mov	w0,PR2				; za sledeci T1INT
	inc	VGATASK			; priprema za main frame
;				

	bclr	IFS0,#T1IF

	defsound

	horsync
	repeat	#2048-HORPOS-1	; HOR POS
	nop			; ovde preskace jedan hor sync

	mov	#FRAME,w0
	mov.b	WREG,VGALATCH
	nop
	nop
	nop
	nop
	clr.b	VGALATCH
	repeat #1589-1
	nop
	mov.b	WREG,VGALATCH
	nop
	nop
	nop
	nop
	clr.b	VGALATCH

	defsound

	pop.s		; /
	bclr	IFS0,#T2IF	; clr int flag
	retfie

;***********************************************************************************
;****************************                          *****************************
;****************************       VGA FRAME          *****************************
;****************************                          *****************************
;***********************************************************************************

taskvga:			; vga frame
;				  ovde je vga main frame (task 1)
	mov	#32*(TEXTPOS-FRAMEPOS)-1,w0	; VREME OD <FRAME> DO <BOTTOM TEXT>
	mov	w0,PR2				; za sledeci T1INT
	inc	VGATASK				; priprema za task 2
;				

	push.d	w4	; \\
	push.d	w6	; \\\
	push.d	w8	; \\\\
	push.d	w10	; \\\\\
	push.d	w12	; \\\\\\
	push	w14	; \\\\\\\
	push	DSRPAG	; \\\\\\\\
	movpag	#1,DSRPAG

	defsound

	bset	FLAG,#0		; handshaking
	inc	VGAUNIFORM
	bclr	FLAG,#2		; clr: row 1, set: row 2

	call	negative_line	; negative line (priprema prve linije)

	clr	w10		; row counter 0...239
	mov	#VGALATCH,w13	; video out port addr
	mov	#480,w11	; row loop counter 480...1 (unazad broji svaku VGA liniju)
	mov	#BKGND-SPRITEBUF1,w12	; offset za bkgnd za liniju 0, 2, 4... (za 1, 3, 5... ce biti umanjen za 190)
;	horsync
9:			   ; \

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
;				priprema reda (do kraja pripreme 503 T max)	(do not trash w10-w14 !!!)	
;_______________________________________________________________________________________________________________

	btsc	FLAG,#2
	bra	phase2

;														
;				prva faza pripreme: pravljenje liste za kopiranje				
;														

; SPRITELIST+0: Xpos, +2: Ypos, +4: sirina sprajta, +6: visina sprajta, +8: offset, +10: page
; COPYLIST+0: sprite addr in rom, +2: sprite page in rom, +4: sprite buffer dst addr, +6: width/4

	mov	#SPRITEBUF2,w0		; current line 0, 2, 4 preparing for line 1, 3, 5...
	btsc	w10,#0		; w10 je brojac linija 0...239 (broji parove VGA linija)
	mov	#SPRITEBUF1,w0		; current line 1, 3, 5 preparing for line 0, 2, 4...
	repeat	#95-1
	clr	[w0++]		; brise sprite buffer
	clr	COPYLISTCOUNT
	mov	#380/2-1,w8		; X pos limit - 1

	mov	#SPRITELIST,w3
	mov	#COPYLIST,w4
	mov	#20,w5		; loop count
5:			; \
	asr	[w3],w7		; +0   X pos sprajta (od leve ivice)
	cpbgt	w7,w8,van1	; bypass if Xpos >= Xpos limit
				; ASR jer je na ulazu bila sirina u pikselima, a trazi se u bajtovima
	bclr	w7,#0		; X has to be word aligned

	mov	[w3+2],w0	; +2   Y pos sprajta
	sub	w10,w0,w1	; linecount-Ypos ---> w1 (taj red piksela treba kopirati)
	bra	ov,van1		; jer taj sprajt nije na toj liniji (ispod te linije je)

	mov	[w3+6],w0	; +6   visina sprajta
	cp	w1,w0		; (red piksela koji se kopira) - visina sprajta
	bra	c,van1		; jer taj sprajt nije na toj liniji (iznad te linije je)

	mov	[w3+4],w6	; +4   sirina sprajta
	lsr	w6,w6
	mulw.uu	w6,w1,w2	; w2 offset od pocetka sprajta u romu do pocetka reda koji treba kopirati
	mov	[w3+8],w0	; +8   adresa sprajta u romu
	add	w0,w2,w2	; source psvoffset

	mov	#SPRITEBUF2,w1			; current line 0, 2, 4 preparing for line 1, 3, 5...
	btsc	w10,#0			; w10 je brojac linija 0...239 (broji parove VGA linija)
	mov	#SPRITEBUF1,w1			; current line 1, 3, 5 preparing for line 0, 2, 4...
	add	w7,w1,w1	; dst (adresa u ramu na koju treba kopirati)

	mov	w2,[w4++]	; sprite addr in rom
	mov	[w3+10],w0	; +10   stranica na kojoj je sprajt u romu
	mov	w0,[w4++]	; sprite page in rom
	mov	w1,[w4++]	; sprite buffer dst addr

	cp	w5,#22-specsprites ; DODATAK 07.11.14.
	ifnc		; DODATAK 07.11.14. ako je w5 manje od #22-specsprites
	bset	w6,#15	; DODATAK 07.11.14. bice setovan bit #14 na sprajtovima 17,18,19,20, koji ne zaklanjaju ostale

	lsr	w6,[w4++]	; sprite width/4 (jer ce kopiranje biti 16-bitno)
	inc	COPYLISTCOUNT
van1:
	add	#12,w3
	dec	w5,w5
	bra	nz,5b	; /

	bra	pripexit

;														
;				druga faza pripreme: kopiranje							
;														

; SPRITELIST+0: Ypos, +2: Xpos, +4: visina sprajta, +6: sirina sprajta, +8: offset, +10: page
; COPYLIST+0: sprite addr in rom, +2: sprite page in rom, +4: sprite buffer dst addr, +6: width/4
phase2:
	mov	COPYLISTCOUNT,w5
	cp0	w5
	bra	z,pripexit
	mov	#COPYLIST,w4
5:			; \
	mov	[w4++],w2	; sprite addr in rom
	mov	[w4++],w3	; sprite page in rom
	mov	[w4++],w1	; sprite buffer dst addr
	dec	[w4++],w6	; sprite width/4-1
	btsc	w6,#14		; DODATAK 07.11.14.
	bra	specmask	; DODATAK 07.11.14. ovo je spec sprite koji ne zaklanja ostale

	movpag	w3,DSRPAG
	repeat	w6
	mov	[w2++],[w1++]	; *** ROM ---> SPRITEBUF ***
	movpag	#1,DSRPAG
spec.ret:
	dec	w5,w5
	bra	nz,5b	; /
pripexit:
	bra	nospecmask
;	
specmask:	; DODATAK 07.11.14: Poslednja 4 sprajta (17,18,19,20) trose puno vremena, ali ne zaklanjaju ostale

	mov	#0xff,w7	; dummy mask
	and	#0x3ff,w6	; clr bit #14
	inc	w6,w6		; restore full width in w6
	
	movpag	w3,DSRPAG
6:
	mov	[w2++],w8	; 5T

	and	w8,w7,w0
	ifnz
	mov.b	w0,[w1]		; *** ROM ---> SPRITEBUF ***
	inc	w1,w1	

	lsr	w8,#8,w0
	ifnz
	mov.b	w0,[w1]		; *** ROM ---> SPRITEBUF ***
	inc	w1,w1	

	dec	w6,w6
	bra	nz,6b

	movpag	#1,DSRPAG
	bra	spec.ret
nospecmask:

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
;				kraj pripreme									
;_______________________________________________________________________________________________________________
;														
;				pocinje slika									
;_______________________________________________________________________________________________________________

	mov	#0b01010101,w14
	mov	#0b10101010,w7

	mov	#SPRITEBUF1-1,w3	; source sprite buffer for line 0, 2, 4...
	btsc	w10,#0		; w10 je brojac linija 0...239 (broji parove VGA linija)
	mov	#SPRITEBUF2-1,w3	; source sprite buffer for line 1, 3, 5...
	bclr	IFS0,#T1IF
	horsync
.rept 190	;	  \\
	and.b	w14,[++w3],w0	; get (spritebuf)
	ifz
	mov.b	[w3+w12],w0	; if (spritebuf)=0, get (bkgnd)
	mov.b	w0,[w13]	; * PIXEL OUT

	and.b	w7,[w3],w0	; get (spritebuf)
	ifz
	mov.b	[w3+w12],w0	; if (spritebuf)=0, get (bkgnd)
	lsr.b	w0,[w13]	; * PIXEL OUT
.endr		;	   //
	nop
	nop
	nop
	clr.b	[w13]		; black pixel

	btsc	FLAG,#2
	inc	w10,w10		; pri svakom drugom prolazu uvecava linecount

	btss	FLAG,#2
	bra	1f		; korekcija offseta se ne vrsi posle 1. malog reda

	btsc	w10,#0
	sub	#86,w12		; izmedju 1. i 2. velikog reda umanjuje offset za medjuprostor2 jer je w3 toliko veci
	btss	w10,#0
	add	#86,w12		; izmedju 2. i 1. velikog reda vraca offset na stari nivo
1:
	dec	w11,w0
	and	#3,w0
	ifz
	add	#2*190,w12	; pri svakom 4.prolazu uvecava offset za 2*190 (jer je w3 veci za 190 u svakom drugom)

	btg	FLAG,#2

	defsound

	dec	w11,w11
	bra	nz,9b	   ; /


;	mov	#0b0011111100111111,w0
;	mov	#BKGND-2,w1
;	mov	w0,[w1]

;    clrled
;			
vgaexit:
	defsound

	pop	DSRPAG	; ////////
	pop	w14	; ///////
	pop.d	w12	; //////
	pop.d	w10	; /////
	pop.d	w8	; ////
	pop.d	w6	; ///
	pop.d	w4	; //

	pop.s		; /
	bclr	IFS0,#T2IF	; clr int flag
	retfie

;***********************************************************************************
;****************************                          *****************************
;****************************    VERT EXTRA TIMING     *****************************
;****************************       BOTTOM TEXT        *****************************
;****************************                          *****************************
;***********************************************************************************

.macro	outpix	cgbit
	mov	w2,w10		; bkgnd color ----> w10
	btsc	[w0],#\cgbit
	mov	w1,w10		; char color ----> w10
	mov.b	w10,[w5]	; ----> write pixel to VGALATCH
.endm

tasktext:
;				  ovde je bottom text (task 2)
	mov	#32*(625-TEXTPOS)-1,w0	; VREME OD <MARKER> do <FRAME>
	mov	w0,PR2				; za sledeci T1INT
	clr	VGATASK				; za sledeci T1INT
;				

	push.d	w4	; \\
	push.d	w6	; \\\
	push.d	w8	; \\\\
	push.d	w10	; \\\\\
	push	DSRPAG	; \\\\\\

	defsound

			; prvo pravi chargen u ramu
	mov	#FIELD,w3	; dst ptr
	movpag	#psvpage(chg7x10),DSRPAG
	mov	#psvoffset(chg7x10),w5
	repeat	#320-1
	mov	[w5++],[w3++]	; PSV
;						

	mov	#FIELD-320,w4	; char gen start + row counter
	mov	#VGALATCH,w5	; constant
	bclr	IFS0,#T1IF

	call	emptyline
	call	emptyline
	call	emptyline
9:			   ; \
	defsound
;		
	mov	#BOTTOMTXT,w3	; txt ptr
	horsync
.rept 39
	mov	[w3++],w6	; read char+param
	lsr	w6,#9,w2	; w2 bkgnd color u  #6 #4 #2 #0
	lsr	w6,#8,w1	; w1 char color u  #6 #4 #2 #0
	and	#0xff,w6	; w6 ASCII char
	mulw.uu	w6,#10,w6	; w6 char*10
	mov.b	w2,[w5]	      ; bkgnd color ----> VGALATCH
	add	w6,w4,w0	; w0 chargen char row addr
	outpix	6
	outpix	5
	outpix	4
	outpix	3
	outpix	2
	outpix	1
	outpix	0
	nop
	nop
	nop
	mov.b	w2,[w5]	      ; bkgnd color ----> VGALATCH
.endr
	nop
	nop
	nop
	clr.b	[w5]		; black ----> VGALATCH
;    
	mov	#BOTTOMTXT,w3	; txt ptr
	horsync
.rept 39
	mov	[w3++],w6	; read char+param
	lsr	w6,#9,w2	; w2 bkgnd color u  #6 #4 #2 #0
	lsr	w6,#8,w1	; w1 char color u  #6 #4 #2 #0
	and	#0xff,w6	; w6 ASCII char
	mulw.uu	w6,#10,w6	; w6 char*10
	mov.b	w2,[w5]	      ; bkgnd color ----> VGALATCH
	add	w6,w4,w0	; w0 chargen char row addr
	outpix	6
	outpix	5
	outpix	4
	outpix	3
	outpix	2
	outpix	1
	outpix	0
	nop
	nop
	nop
	mov.b	w2,[w5]	      ; bkgnd color ----> VGALATCH
.endr
	nop
	nop
	nop
	clr.b	[w5]		; black ----> VGALATCH
;		
	mov	#BOTTOMTXT,w3	; txt ptr
	horsync
.rept 39
	mov	[w3++],w6	; read char+param
	lsr	w6,#9,w2	; w2 bkgnd color u  #6 #4 #2 #0
	lsr	w6,#8,w1	; w1 char color u  #6 #4 #2 #0
	and	#0xff,w6	; w6 ASCII char
	mulw.uu	w6,#10,w6	; w6 char*10
	mov.b	w2,[w5]	      ; bkgnd color ----> VGALATCH
	add	w6,w4,w0	; w0 chargen char row addr
	outpix	6+8
	outpix	5+8
	outpix	4+8
	outpix	3+8
	outpix	2+8
	outpix	1+8
	outpix	0+8
	nop
	nop
	nop
	mov.b	w2,[w5]	      ; bkgnd color ----> VGALATCH
.endr
	nop
	nop
	nop
	clr.b	[w5]		; black ----> VGALATCH
;    
	mov	#BOTTOMTXT,w3	; txt ptr
	horsync
.rept 39
	mov	[w3++],w6	; read char+param
	lsr	w6,#9,w2	; w2 bkgnd color u  #6 #4 #2 #0
	lsr	w6,#8,w1	; w1 char color u  #6 #4 #2 #0
	and	#0xff,w6	; w6 ASCII char
	mulw.uu	w6,#10,w6	; w6 char*10
	mov.b	w2,[w5]	      ; bkgnd color ----> VGALATCH
	add	w6,w4,w0	; w0 chargen char row addr
	outpix	6+8
	outpix	5+8
	outpix	4+8
	outpix	3+8
	outpix	2+8
	outpix	1+8
	outpix	0+8
	nop
	nop
	nop
	mov.b	w2,[w5]	      ; bkgnd color ----> VGALATCH
.endr
	nop
	nop
	nop
	clr.b	[w5]		; black ----> VGALATCH
;		
	inc2	w4,w4		; adv row counter
	mov	#FIELD-320+10,w0
	cp	w4,w0		; test for loop end (10 passes)
	bra	nc,9b	   ; /

	call	emptyline
	call	emptyline
	call	emptyline
;	
t5exit:
	defsound

	pop	DSRPAG	; //////
	pop.d	w10	; /////
	pop.d	w8	; ////
	pop.d	w6	; ///
	pop.d	w4	; //

	pop.s		; /
	bclr	IFS0,#T2IF	; clr int flag
	retfie

;							
;							
;							
emptyline:
	mov	#VGALATCH,w5	; constant
	mov	#BOTTOMTXT,w3	; txt ptr
	horsync
.rept 39
	mov	[w3++],w6	; read char+param
	lsr	w6,#9,w2	; w2 bkgnd color u  #6 #4 #2 #0
	nop
	nop
	nop
	mov.b	w2,[w5]	      ; bkgnd color ----> VGALATCH
	repeat	#32-1
	nop
.endr
	nop
	nop
	nop
	clr.b	[w5]		; black ----> VGALATCH
	return




